@@ -20,10 +20,6 @@ logger = logging.getLogger(__name__)
 # KATCP Type Classes
 #
 
-# XXX how about a mapping from python types -> kattypes, so creating
-#     a sensor would not require importing kattypes
-
-
 class KatcpType(object):
     """Class representing a KATCP type.
 
@@ -118,7 +114,10 @@ class KatcpType(object):
         if packed_value is None:
             value = self.get_default()
         else:
-            value = self.decode(packed_value, major)
+            try:
+                value = self.decode(packed_value, major)
+            except Exception:
+                raise
         if value is not None:
             self.check(value, major)
         return value
@@ -162,6 +161,16 @@ class Int(KatcpType):
             raise ValueError("Integer %d is higher than maximum %d."
                 % (value, self._max))
 
+class GaussianInt(KatcpType):
+    """
+    Complex integer numbers as per NRC-KAT7-6.0-IFCE-002 (KATCP guidelines) A.5
+    """
+
+    name = 'gausian_integer'
+
+    def encode(self, value, major):
+        pass
+        
 
 class Float(KatcpType):
     """The KATCP float type.
@@ -571,7 +580,6 @@ class Parameter(object):
             raise FailReply("Error in parameter %s (%s): %s" %
                             (self.position, self.name, message))
 
-
 ## request, return_reply and inform method decorators
 #
 
@@ -814,7 +822,7 @@ def send_reply(*types, **options):
     Examples
     --------
     >>> class MyDevice(DeviceServer):
-    ...     @send_reply('myreq', Int(), Float())
+    ...     @send_reply(Int(), Float())
     ...     def my_callback(self, req):
     ...         return (req, "ok", 5, 2.0)
     ...
